from unittest.mock import ANY, patch

import pytest
from django.core.management import call_command

from rele import Worker


class TestRunReleCommand:
    @pytest.fixture(autouse=True)
    def worker_wait_forever(self):
        with patch.object(
            Worker, "_wait_forever", return_value=None, autospec=True
        ) as p:
            yield p

    def test_calls_worker_start_and_setup_when_runrele(self, mock_worker):
        call_command("runrele")

        mock_worker.assert_called_with(
            [], "rele-test", ANY, "europe-west1", 60, 2, None
        )
        mock_worker.return_value.run_forever.assert_called_once_with()

    def test_prints_warning_when_conn_max_age_not_set_to_zero(
        self, mock_worker, capsys, settings
    ):
        settings.DATABASES = {"default": {"CONN_MAX_AGE": 1}}
        call_command("runrele")

        out, err = capsys.readouterr()
        assert (
            "WARNING: settings.CONN_MAX_AGE is not set to 0. "
            "This may result in slots for database connections to "
            "be exhausted." in err
        )
        mock_worker.assert_called_with(
            [], "rele-test", ANY, "europe-west1", 60, 2, None
        )
        mock_worker.return_value.run_forever.assert_called_once_with()



import json
import logging
import time
from collections.abc import Iterable
from inspect import getfullargspec, getmodule

from .middleware import run_middleware_hook

logger = logging.getLogger(__name__)


class Subscription:
    """The Subscription class

    In addition to using the ``@sub`` decorator, it is possible to subclass
    the Subscription.

    For example::

        from rele import Subscription

        class DoSomethingSub(Subscription):
            topic = 'photo-uploaded'

            def __init__(self):
                self._func = self.callback_func
                super().__init__(self._func, self.topic)

            def callback_func(self, data, **kwargs):
                print(data["id"])

    If ``rele-cli run`` is used, the ``DoSomethingSub`` will be a valid subscription
    and registered on Google Cloud.

    """

    def __init__(
        self,
        func,
        topic,
        prefix="",
        suffix="",
        filter_by=None,
        backend_filter_by=None,
        retry_policy=None,
    ):
        self._validate_filter_by(filter_by)

        self._func = func
        self.topic = topic
        self._prefix = prefix
        self._suffix = suffix
        self._filters = self._init_filters(filter_by)
        self.backend_filter_by = backend_filter_by
        self.retry_policy = retry_policy

    def _validate_filter_by(self, filter_by):
        if filter_by and not (
            callable(filter_by)
            or (
                isinstance(filter_by, Iterable)
                and all(callable(filter) for filter in filter_by)
            )
        ):
            raise ValueError("Filter_by must be a callable or a list of callables.")

    def _init_filters(self, filter_by):
        if isinstance(filter_by, Iterable):
            return filter_by
        elif filter_by:
            return [filter_by]

        return None

    @property
    def name(self):
        name_parts = [self._prefix, self.topic, self._suffix]
        return "-".join(filter(lambda x: x, name_parts))

    @property
    def prefix(self):
        return self._prefix

    def set_prefix(self, prefix):
        self._prefix = prefix

    @property
    def filter_by(self):
        return self._filters

    def set_filters(self, filter_by):
        self._filters = filter_by

    def __call__(self, data, **kwargs):
        if "published_at" in kwargs:
            kwargs["published_at"] = float(kwargs["published_at"])

        if self._any_filter_returns_false(kwargs):
            return

        return self._func(data, **kwargs)

    def __str__(self):
        return f"{self.name} - {self._func.__name__}"

    def _any_filter_returns_false(self, kwargs):
        if not self._filters:
            return False

        return not all(filter(kwargs) for filter in self._filters)


class Callback:
    def __init__(self, subscription, suffix=None):
        self._subscription = subscription
        self._suffix = suffix

    def __call__(self, message):
        run_middleware_hook("pre_process_message", self._subscription, message)
        start_time = time.time()

        try:
            data = json.loads(message.data.decode("utf-8"))
        except json.JSONDecodeError as e:
            message.ack()
            run_middleware_hook(
                "post_process_message_failure",
                self._subscription,
                e,
                start_time,
                message,
            )
            run_middleware_hook("post_process_message")
            return

        try:
            res = self._subscription(data, **dict(message.attributes))
        except Exception as e:
            run_middleware_hook(
                "post_process_message_failure",
                self._subscription,
                e,
                start_time,
                message,
            )
        else:
            message.ack()
            run_middleware_hook(
                "post_process_message_success",
                self._subscription,
                start_time,
                message,
            )
            return res
        finally:
            run_middleware_hook("post_process_message")


def sub(
    topic,
    prefix=None,
    suffix=None,
    filter_by=None,
    backend_filter_by=None,
    retry_policy=None,
):
    """Decorator function that makes declaring a PubSub Subscription simple.

    The Subscriber returned will automatically create and name
    the subscription for the topic.
    The subscription name will be the topic name prefixed by the project name.

    For example, if the topic name to subscribe too is `lets-tell-everyone`,
    the subscriber will be named `project-name-lets-tell-everyone`.

    Additionally, if a `suffix` param is added, the subscriber will be
    `project-name-lets-tell-everyone-my-suffix`.

    It is recommended to add `**kwargs` to your `sub` function. This will allow
    message attributes to be sent without breaking the subscriber
    implementation.

    Usage::

        @sub(topic='lets-tell-to-alice', prefix='shop')
        def bob_purpose(data, **kwargs):
             pass

        @sub(topic='lets-tell-everyone', suffix='sub1')
        def purpose_1(data, **kwargs):
             pass

        @sub(topic='lets-tell-everyone', suffix='sub2')
        def purpose_2(data, **kwargs):
             pass

        @sub(topic='photo-updated',
             filter_by=lambda **attrs: attrs.get('type') == 'landscape')
        def sub_process_landscape_photos(data, **kwargs):
            pass

    :param topic: string The topic that is being subscribed to.
    :param prefix: string An optional prefix to the subscription name.
                   Useful to namespace your subscription with your project name
    :param suffix: string An optional suffix to the subscription name.
                   Useful when you have two subscribers in the same project
                   that are subscribed to the same topic.
    :param filter_by: Union[function, list] An optional function or tuple of
                      functions that filters the messages to be processed by
                      the sub regarding their attributes.
    :param retry_policy: obj :class:`~rele.retry_policy.RetryPolicy`
    :return: :class:`~rele.subscription.Subscription`
    """

    def decorator(func):
        args_spec = getfullargspec(func)
        if len(args_spec.args) != 1 or not args_spec.varkw:
            raise RuntimeError(
                f"Subscription function {func.__module__}.{func.__name__} is not valid. "
                "The function must have one argument and accept keyword arguments."
            )

        if getmodule(func).__name__.split(".")[-1] != "subs":
            logger.warning(
                f"Subscription function {func.__module__}.{func.__name__} is "
                "outside a subs module that will not be discovered."
            )

        return Subscription(
            func=func,
            topic=topic,
            prefix=prefix,
            suffix=suffix,
            filter_by=filter_by,
            backend_filter_by=backend_filter_by,
            retry_policy=retry_policy,
        )

    return decorator



from rele.middleware import BaseMiddleware


class UnrecoverableException(Exception):
    pass


class UnrecoverableMiddleWare(BaseMiddleware):
    def post_process_message_failure(self, subscription, err, start_time, message):
        if isinstance(err, UnrecoverableException):
            message.ack()



from unittest.mock import patch

import pytest
from django.core.management import call_command

from rele.subscription import sub


@sub(topic="some-cool-topic", prefix="rele")
def sub_stub(data, **kwargs):
    return data["id"]


@sub(topic="some-fancy-topic")
def sub_fancy_stub(data, **kwargs):
    return data["id"]


@sub(topic="published-time-type")
def sub_published_time_type(data, **kwargs):
    return f'{type(kwargs["published_at"])}'


def landscape_filter(**kwargs):
    return kwargs.get("type") == "landscape"


@sub(topic="photo-updated", filter_by=landscape_filter)
def sub_process_landscape_photos(data, **kwargs):
    return f'Received a photo of type {kwargs.get("type")}'


@pytest.fixture()
def mock_discover_subs():
    affected_path = (
        "rele.management.commands.showsubscriptions" ".discover_subs_modules"
    )
    with patch(affected_path, return_value=[__name__]) as mock:
        yield mock


class TestShowSubscriptions:
    def test_prints_table_when_called(self, capfd, mock_discover_subs):
        call_command("showsubscriptions")

        mock_discover_subs.assert_called_once()
        captured = capfd.readouterr()
        assert (
            captured.out
            == """Topic                Subscriber(s)         Sub
-------------------  --------------------  ----------------------------
photo-updated        photo-updated         sub_process_landscape_photos
published-time-type  published-time-type   sub_published_time_type
some-cool-topic      rele-some-cool-topic  sub_stub
some-fancy-topic     some-fancy-topic      sub_fancy_stub
"""
        )



import json
import logging
import os
import time
from concurrent.futures import TimeoutError

import google.auth
from google.api_core import exceptions
from google.cloud import pubsub_v1
from google.cloud.pubsub_v1.types import FieldMask
from google.protobuf import duration_pb2
from google.pubsub_v1 import MessageStoragePolicy
from google.pubsub_v1 import RetryPolicy as GCloudRetryPolicy

from rele.middleware import run_middleware_hook

logger = logging.getLogger(__name__)

USE_EMULATOR = True if os.environ.get("PUBSUB_EMULATOR_HOST") else False
DEFAULT_ENCODER_PATH = "json.JSONEncoder"
DEFAULT_ACK_DEADLINE = 60
DEFAULT_BLOCKING = False


def get_google_defaults():
    try:
        credentials, project = google.auth.default()
        return credentials, project
    except google.auth.exceptions.DefaultCredentialsError:
        return None, None


class Subscriber:
    """The Subscriber Class.

    For convenience, this class wraps the creation and consumption of a topic
    subscription.

    :param gc_project_id: str :ref:`settings_project_id` .
    :param credentials: obj :meth:`~rele.config.Config.credentials`.
    :param message_storage_policy: str Region to store the messages
    :param default_ack_deadline: int Ack Deadline defined in settings
    :param default_retry_policy: RetryPolicy Rele's RetryPolicy defined in settings
    """

    def __init__(
        self,
        gc_project_id,
        credentials,
        message_storage_policy,
        default_ack_deadline=None,
        default_retry_policy=None,
    ):
        self._gc_project_id = gc_project_id
        self._ack_deadline = default_ack_deadline or DEFAULT_ACK_DEADLINE
        self.credentials = credentials if not USE_EMULATOR else None
        self._message_storage_policy = message_storage_policy
        self._client = pubsub_v1.SubscriberClient(credentials=credentials)
        self._retry_policy = default_retry_policy

    def update_or_create_subscription(self, subscription):
        """Handles creating the subscription when it does not exists or updates it
        if the subscription contains any parameter that allows it.

        This makes it easier to deploy a worker and forget about the
        subscription side of things. The subscription must
        have a topic to subscribe to. Which means that the topic must be
        created manually before the worker is started.

        :param subscription: obj :class:`~rele.subscription.Subscription`.
        """
        subscription_path = self._client.subscription_path(
            self._gc_project_id, subscription.name
        )
        topic_path = self._client.topic_path(self._gc_project_id, subscription.topic)

        try:
            self._create_subscription(subscription_path, topic_path, subscription)
        except exceptions.NotFound:
            logger.warning(
                "Cannot subscribe to a topic that does not exist."
                f"Creating {topic_path}..."
            )
            topic = self._create_topic(topic_path)
            logger.info(f"Topic {topic.name} created.")
            self._create_subscription(subscription_path, topic_path, subscription)
        except exceptions.AlreadyExists:
            self._update_subscription(subscription_path, topic_path, subscription)

    def _create_topic(self, topic_path):
        publisher_client = pubsub_v1.PublisherClient(credentials=self.credentials)
        return publisher_client.create_topic(
            request={
                "name": topic_path,
                "message_storage_policy": MessageStoragePolicy(
                    {"allowed_persistence_regions": [self._message_storage_policy]}
                ),
            }
        )

    def _create_subscription(self, subscription_path, topic_path, subscription):
        request = {
            "name": subscription_path,
            "topic": topic_path,
            "ack_deadline_seconds": self._ack_deadline,
        }

        if subscription.backend_filter_by:
            request["filter"] = subscription.backend_filter_by

        retry_policy = subscription.retry_policy or self._retry_policy

        if retry_policy:
            request["retry_policy"] = self._build_gcloud_retry_policy(retry_policy)

        self._client.create_subscription(request=request)

    def _update_subscription(self, subscription_path, topic_path, subscription):
        retry_policy = subscription.retry_policy or self._retry_policy

        if not retry_policy:
            return

        update_mask = FieldMask(paths=["retry_policy"])

        client_retry_policy = self._build_gcloud_retry_policy(retry_policy)

        subscription = pubsub_v1.types.Subscription(
            name=subscription_path,
            topic=topic_path,
            retry_policy=client_retry_policy,
        )

        self._client.update_subscription(
            request={"subscription": subscription, "update_mask": update_mask}
        )

    def _build_gcloud_retry_policy(self, rele_retry_policy):
        minimum_backoff = duration_pb2.Duration(
            seconds=rele_retry_policy.minimum_backoff
        )
        maximum_backoff = duration_pb2.Duration(
            seconds=rele_retry_policy.maximum_backoff
        )

        return GCloudRetryPolicy(
            minimum_backoff=minimum_backoff, maximum_backoff=maximum_backoff
        )

    def consume(self, subscription_name, callback, scheduler):
        """Begin listening to topic from the SubscriberClient.

        :param subscription_name: str Subscription name
        :param callback: Function which act on a topic message
        :param scheduler: `Thread pool-based scheduler. <https://googleapis.dev/python/pubsub/latest/subscriber/api/scheduler.html?highlight=threadscheduler#google.cloud.pubsub_v1.subscriber.scheduler.ThreadScheduler>`_  # noqa
        :return: `Future <https://googleapis.github.io/google-cloud-python/latest/pubsub/subscriber/api/futures.html>`_  # noqa
        """
        subscription_path = self._client.subscription_path(
            self._gc_project_id, subscription_name
        )
        return self._client.subscribe(
            subscription_path, callback=callback, scheduler=scheduler
        )


class Publisher:
    """The Publisher Class

    Wraps the Google Cloud Publisher Client and handles encoding of the data.

    It is important that this class remains a Singleton class in the process.
    Otherwise, a memory leak will occur. To avoid this, it is strongly
    recommended to use the :meth:`~rele.publishing.publish` method.

    If the setting `USE_EMULATOR` evaluates to True, the Publisher Client will
    not have any credentials assigned.

    :param gc_project_id: string Google Cloud Project ID.
    :param credentials: string Google Cloud Credentials.
    :param encoder: A valid `json.encoder.JSONEncoder subclass <https://docs.python.org/3/library/json.html#json.JSONEncoder>`_  # noqa
    :param timeout: float, default :ref:`settings_publisher_timeout`
    :param blocking: boolean, default None falls back to :ref:`settings_publisher_blocking`
    """

    def __init__(self, gc_project_id, credentials, encoder, timeout, blocking=None):
        self._gc_project_id = gc_project_id
        self._timeout = timeout
        self._blocking = blocking
        self._encoder = encoder
        if USE_EMULATOR:
            self._client = pubsub_v1.PublisherClient()
        else:
            self._client = pubsub_v1.PublisherClient(credentials=credentials)

    def publish(
        self, topic, data, blocking=None, timeout=None, raise_exception=True, **attrs
    ):
        """Publishes message to Google PubSub topic.

        Usage::

            publisher = Publisher()
            publisher.publish('topic_name', {'foo': 'bar'})

        By default, this method is non-blocking, meaning that the method does
        not wait for the future to be returned.

        If you would like to wait for the future so you can track the message
        later, you can:

        Usage::

            publisher = Publisher()
            future = publisher.publish('topic_name', {'foo': 'bar'}, blocking=True, timeout=10.0) # noqa

        However, it should be noted that using `blocking=True` may incur a
        significant performance hit.

        In addition, the method adds a timestamp `published_at` to the
        message attrs using `epoch floating point number
        <https://docs.python.org/3/library/time.html#time.time>`_.

        :param topic: string topic to publish the data.
        :param data: dict with the content of the message.
        :param blocking: boolean, default None falls back to :ref:`settings_publisher_blocking`
        :param timeout: float, default None falls back to :ref:`settings_publisher_timeout`
        :param raise_exception: boolean. If True, exceptions coming from PubSub will be raised
        :param attrs: additional string parameters to be published.
        :return: `Future <https://googleapis.github.io/google-cloud-python/latest/pubsub/subscriber/api/futures.html>`_  # noqa
        """
        if blocking is None:
            blocking = self._blocking

        attrs["published_at"] = str(time.time())
        run_middleware_hook("pre_publish", topic, data, attrs)
        payload = json.dumps(data, cls=self._encoder).encode("utf-8")
        topic_path = self._client.topic_path(self._gc_project_id, topic)
        future = self._client.publish(topic_path, payload, **attrs)
        if not blocking:
            return future

        try:
            future.result(timeout=timeout or self._timeout)
        except TimeoutError as e:
            run_middleware_hook("post_publish_failure", topic, e, data)
            if raise_exception:
                raise e
        else:
            run_middleware_hook("post_publish_success", topic, data, attrs)

            # DEPRECATED
            run_middleware_hook("post_publish", topic)

        return future



from django.core.management import BaseCommand
from tabulate import tabulate

from rele.config import load_subscriptions_from_paths
from rele.management.discover import discover_subs_modules


class Command(BaseCommand):
    help = "List information about Pub/Sub subscriptions registered using Relé."

    def handle(self, *args, **options):
        headers = ["Topic", "Subscriber(s)", "Sub"]

        subscription_paths = discover_subs_modules()
        subs = sorted(
            load_subscriptions_from_paths(subscription_paths),
            key=lambda sub: sub.topic,
        )
        sub_data = [(sub.topic, sub.name, sub._func.__name__) for sub in subs]

        self.stdout.write(tabulate(sub_data, headers=headers))



from rele.__main__ import run_worker


class TestReleCli:
    def test_rele_cli_run(self, mock_worker):
        run_worker("tests.settings", ["sample_pypi_package.subs"])
        worker_subscriptions_argument = mock_worker.mock_calls[0].args[0]
        topic_names = [sub.name for sub in worker_subscriptions_argument]

        assert "rele-topic-from-third-party-package" in topic_names

    def test_ignores_non_valid_third_party_subs(self, mock_worker):
        run_worker("tests.settings", ["sample_pypi_package.no_subs"])

        mock_worker.assert_called()



import importlib
import os

from google.oauth2 import service_account

from .client import (
    DEFAULT_ACK_DEADLINE,
    DEFAULT_BLOCKING,
    DEFAULT_ENCODER_PATH,
    get_google_defaults,
)
from .middleware import default_middleware, register_middleware
from .publishing import init_global_publisher
from .subscription import Subscription


class Config:
    """Configuration class.

    The settings.RELE dictionary will be parsed into an easily
    accessible object containing all the necessary constants.

    If no middleware is set, the *default_middleware* will be added.

    :param setting: dict :ref:`settings <_settings>`
    """

    def __init__(self, setting):
        self._gc_project_id = setting.get("GC_PROJECT_ID")
        self.gc_credentials_path = setting.get("GC_CREDENTIALS_PATH")
        self.gc_storage_region = setting.get("GC_STORAGE_REGION", "europe-west1")
        self.app_name = setting.get("APP_NAME")
        self.sub_prefix = setting.get("SUB_PREFIX")
        self.middleware = setting.get("MIDDLEWARE", default_middleware)
        self.ack_deadline = setting.get(
            "ACK_DEADLINE",
            os.environ.get("DEFAULT_ACK_DEADLINE", DEFAULT_ACK_DEADLINE),
        )
        self.publisher_blocking = setting.get("PUBLISHER_BLOCKING", DEFAULT_BLOCKING)
        self._encoder_path = setting.get("ENCODER_PATH", DEFAULT_ENCODER_PATH)
        self.publisher_timeout = setting.get("PUBLISHER_TIMEOUT", 3.0)
        self.threads_per_subscription = setting.get("THREADS_PER_SUBSCRIPTION", 2)
        self.filter_by = setting.get("FILTER_SUBS_BY")
        self._credentials = None
        self.retry_policy = setting.get("DEFAULT_RETRY_POLICY")

    @property
    def encoder(self):
        module_name, class_name = self._encoder_path.rsplit(".", 1)
        module = importlib.import_module(module_name)
        return getattr(module, class_name)

    @property
    def credentials(self):
        if self.gc_credentials_path:
            self._credentials = service_account.Credentials.from_service_account_file(
                self.gc_credentials_path
            )
        else:
            credentials, project_id = get_google_defaults()
            self._credentials = credentials
            if not self._gc_project_id:
                self._gc_project_id = project_id
        return self._credentials

    @property
    def gc_project_id(self):
        if self._gc_project_id:
            return self._gc_project_id
        elif self.credentials:
            return self.credentials.project_id
        else:
            return None


def setup(setting=None, **kwargs):
    if setting is None:
        setting = {}

    config = Config(setting)
    init_global_publisher(config)
    register_middleware(config, **kwargs)
    return config


def is_same_subscription(a_subscription, another_subscription):
    return id(a_subscription) == id(another_subscription)


def is_subscription_registered(subscriptions, current_subscription):
    registered_subscription = subscriptions.get(current_subscription.name)
    return is_same_subscription(registered_subscription, current_subscription)


def subscription_from_attribute(attribute):
    try:
        if isinstance(attribute, Subscription):
            subscription = attribute
        elif issubclass(attribute, Subscription):
            subscription = attribute()
        else:
            return None
    except TypeError:
        # If attribute is not a class, TypeError is raised when testing issubclass
        return None
    return subscription


def load_subscriptions_from_paths(sub_module_paths, sub_prefix=None, filter_by=None):
    subscriptions = {}
    for sub_module_path in sub_module_paths:
        sub_module = importlib.import_module(sub_module_path)
        for attr_name in dir(sub_module):
            attribute = getattr(sub_module, attr_name)

            subscription = subscription_from_attribute(attribute)
            if not subscription:
                continue

            if sub_prefix and not subscription.prefix:
                subscription.set_prefix(sub_prefix)

            if filter_by and not subscription.filter_by:
                subscription.set_filters(filter_by)

            if is_subscription_registered(subscriptions, subscription):
                continue

            if subscription.name in subscriptions:
                found_subscription = subscriptions[subscription.name]
                raise RuntimeError(
                    f"Duplicate subscription name found: {subscription.name}. Subs "
                    f"{subscription._func.__module__}.{subscription._func.__name__} and "
                    f"{found_subscription._func.__module__}.{found_subscription._func.__name__} collide."
                )

            subscriptions[subscription.name] = subscription
    return list(subscriptions.values())



import importlib
import logging
import pkgutil
from importlib.util import find_spec as importlib_find

logger = logging.getLogger(__name__)


def module_has_submodule(package, module_name):
    """
    See if 'module' is in 'package'.
    Taken from https://github.com/django/django/blob/master/django/utils/module_loading.py#L63
    """
    imported_package = importlib.import_module(package)
    package_name = imported_package.__name__
    package_path = imported_package.__path__
    full_module_name = package_name + "." + module_name
    try:
        return importlib_find(full_module_name, package_path) is not None
    except (ModuleNotFoundError, AttributeError):
        # When module_name is an invalid dotted path, Python raises
        # ModuleNotFoundError.
        return False


def _import_settings_from_path(path):
    if path is not None:
        print(" * Discovered settings: %r" % path)
        return importlib.import_module(path)


def sub_modules(settings_path=None):
    """
    In the current directory, we can traverse all modules and determine if they
    have a settings.py or directory with a subs.py module. If either one of
    those exists, we import it, and return the settings module, and
    paths to the subs file.

    If a settings module is not found, we return None.

    :return: (settings module, List[string: subs module paths])
    """
    module_paths = []
    for f, package, is_package in pkgutil.walk_packages(path=["."]):
        if package == "settings":
            settings_path = package
        if is_package and module_has_submodule(package, "subs"):
            module = package + ".subs"
            module_paths.append(module)
            print(" * Discovered subs module: %r" % module)

    settings = _import_settings_from_path(settings_path)
    return settings, module_paths



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import re
import sys

from setuptools import find_packages, setup


def get_version(*file_paths):
    """Retrieves the version from rele/__init__.py"""
    filename = os.path.join(os.path.dirname(__file__), *file_paths)
    version_file = open(filename).read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


version = get_version("rele", "__init__.py")


if sys.argv[-1] == "tag":
    print("Tagging the version on git: %s" % version)
    os.system('git tag -a %s -m "version %s"' % (version, version))
    os.system("git push --tags")
    sys.exit()

readme = open("README.md").read()

setup(
    name="rele",
    version=version,
    description="""Relé makes integration with Google PubSub easier.""",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="Mercadona Tech",
    author_email="software.online@mercadona.es",
    url="https://github.com/mercadona/rele",
    packages=find_packages(exclude=("tests",)),
    include_package_data=True,
    install_requires=["google-auth", "google-cloud-pubsub>=2.2.0"],
    extras_require={"django": ["django", "tabulate"], "flask": ["flask"]},
    license="Apache Software License 2.0",
    zip_safe=False,
    keywords="rele",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Framework :: Django :: 1.11",
        "Framework :: Django :: 2.0",
        "Framework :: Django :: 2.1",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.0",
        "Framework :: Django :: 3.1",
        "Framework :: Django :: 3.2",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    entry_points={"console_scripts": ["rele-cli=rele.__main__:main"]},
)
